@@ -20,16 +20,20 @@ from optparse import OptionParser, OptionGroup, Option, OptionValueError
 from ConfigParser import SafeConfigParser, NoSectionError
 from copy import copy
 import os.path
+import tempfile
+
 try:
     from gbp.version import gbp_version
 except ImportError:
     gbp_version = "[Unknown version]"
 import gbp.tristate
+from gbp.git import GitRepositoryError, GitRepository
 
 no_upstream_branch_msg = """
 Repository does not have branch '%s' for upstream sources. If there is none see
 file:///usr/share/doc/git-buildpackage/manual-html/gbp.import.html#GBP.IMPORT.CONVERT
-on howto create it otherwise use --upstream-branch to specify it.
+on howto create it or check the --create-missing-branches option. Otherwise,
+use --upstream-branch to specify it.
 """
 
 def expand_path(option, opt, value):
@@ -45,6 +49,27 @@ def check_tristate(option, opt, value):
     else:
         return val
 
+
+def safe_option(f):
+    def _decorator(self, *args, **kwargs):
+        obj = self
+        option_name = kwargs.get('option_name')
+        if not option_name and len(args):
+            option_name = args[0]
+
+        # We're decorating GbpOption not GbpOptionParser
+        if not hasattr(obj, 'valid_options'):
+            if not hasattr(obj, 'parser'):
+                raise ValueError("Can only decorete GbpOptionParser and GbpOptionGroup not %s" % obj)
+            else:
+                obj = obj.parser
+
+        if option_name and not option_name.startswith('no-'):
+            obj.valid_options.append(option_name)
+        return f(self, *args, **kwargs)
+    return _decorator
+
+
 class GbpOption(Option):
     TYPES = Option.TYPES + ('path', 'tristate')
     TYPE_CHECKER = copy(Option.TYPE_CHECKER)
@@ -69,6 +94,7 @@ class GbpOptionParser(OptionParser):
     @type def_config_files: list
     """
     defaults = { 'debian-branch'   : 'master',
+                 'packaging-branch' : 'master',
                  'upstream-branch' : 'upstream',
                  'upstream-tree'   : 'TAG',
                  'pristine-tar'    : 'False',
@@ -83,8 +109,10 @@ class GbpOptionParser(OptionParser):
                  'prebuild'        : '',
                  'postexport'      : '',
                  'postimport'      : '',
+                 'build'           : 'True',
                  'hooks'           : 'True',
                  'debian-tag'      : 'debian/%(version)s',
+                 'debian-tag-msg'  : '%(pkg)s Debian release %(version)s',
                  'upstream-tag'    : 'upstream/%(version)s',
                  'import-msg'      : 'Imported Upstream version %(version)s',
                  'commit-msg'      : 'Update changelog for %(version)s release',
@@ -92,12 +120,11 @@ class GbpOptionParser(OptionParser):
                  'snapshot-number' : 'snapshot + 1',
                  'git-log'         : '--no-merges',
                  'export'          : 'HEAD',
-                 'export-dir'      : '',
                  'overlay'         : 'False',
                  'tarball-dir'     : '',
                  'ignore-new'      : 'False',
                  'ignore-branch'   : 'False',
-                 'meta'            : 'False',
+                 'meta'            : 'True',
                  'meta-closes'     : 'Closes|LP',
                  'full'            : 'False',
                  'id-length'       : '0',
@@ -133,11 +160,18 @@ class GbpOptionParser(OptionParser):
                  'allow-unauthenticated': 'False',
                  'symlink-orig': 'True',
                  'purge': 'True',
+                 'drop': 'False',
+                 'commit': 'False',
+                 'tmp-dir' : '/var/tmp/gbp/',
              }
     help = {
              'debian-branch':
                   ("Branch the Debian package is being developed on, "
                    "default is '%(debian-branch)s'"),
+             'packaging-branch':
+                  ("Branch the packaging is being maintained on, "
+                   "rpm counterpart of the 'debian-branch' option, "
+                   "default is '%(packaging-branch)s'"),
              'upstream-branch':
                   "Upstream branch, default is '%(upstream-branch)s'",
              'upstream-tree':
@@ -146,6 +180,9 @@ class GbpOptionParser(OptionParser):
              'debian-tag':
                   ("Format string for debian tags, "
                    "default is '%(debian-tag)s'"),
+             'debian-tag-msg':
+                  ("Format string for signed debian-tag messages, "
+                   "default is '%(debian-tag-msg)s'"),
              'upstream-tag':
                   ("Format string for upstream tags, "
                    "default is '%(upstream-tag)s'"),
@@ -175,6 +212,8 @@ class GbpOptionParser(OptionParser):
                   "Include the full commit message instead of only the first line, default is '%(full)s'",
              'meta':
                   "Parse meta tags in commit messages, default is '%(meta)s'",
+             'meta-closes':
+                  "Meta tags for the bts close commands, default is '%(meta-closes)s'",
              'ignore-new':
                   "Build with uncommited changes in the source tree, default is '%(ignore-new)s'",
              'ignore-branch':
@@ -228,10 +267,10 @@ class GbpOptionParser(OptionParser):
                   ("Set up tracking for remote branches, "
                    "default is '%(track)s'"),
              'author-is-committer':
-                  ("Use the authors's name also as the comitter's name, "
+                  ("Use the authors's name also as the committer's name, "
                    "default is '%(author-is-committer)s'"),
              'author-date-is-committer-date':
-                  ("Use the authors's date as the comitter's date, "
+                  ("Use the authors's date as the committer's date, "
                    "default is '%(author-date-is-committer-date)s'"),
              'create-missing-branches':
                   ("Create missing branches automatically, "
@@ -242,6 +281,8 @@ class GbpOptionParser(OptionParser):
              'postimport':
                   ("hook run after a successful import, "
                    "default is '%(postimport)s'"),
+             'build':
+                  ("Enable running builder, default is %(build)s"),
              'hooks':
                   ("Enable running all hooks, default is %(hooks)s"),
              'time-machine':
@@ -269,32 +310,44 @@ class GbpOptionParser(OptionParser):
                    "'%(symlink-orig)s'"),
               'purge':
                   "Purge exported package build directory. Default is '%(purge)s'",
+              'drop':
+                  ("In case of 'export' drop the patch-queue branch "
+                   "after export. Default is '%(drop)s'"),
+              'commit':
+                  "commit changes after export, Default is '%(commit)s'",
+              'tmp-dir':
+                  ("Base directory under which temporary directories are "
+                   "created, default is '%(tmp-dir)s'"),
            }
 
     def_config_files = [ '/etc/git-buildpackage/gbp.conf',
                          '~/.gbp.conf',
-                         '.gbp.conf',
-                         'debian/gbp.conf',
-                         '.git/gbp.conf' ]
+                         '%(top_dir)s/.gbp.conf',
+                         '%(top_dir)s/debian/gbp.conf',
+                         '%(git_dir)s/gbp.conf' ]
 
     @classmethod
-    def get_config_files(klass):
+    def get_config_files(klass, no_local=False):
         """
         Get list of config files from the I{GBP_CONF_FILES} environment
         variable.
 
+        @param no_local: don't return the per-repo configuration files
+        @type no_local: C{str}
         @return: list of config files we need to parse
         @rtype: C{list}
 
         >>> conf_backup = os.getenv('GBP_CONF_FILES')
         >>> if conf_backup is not None: del os.environ['GBP_CONF_FILES']
+        >>> homedir = os.path.expanduser("~")
         >>> files = GbpOptionParser.get_config_files()
-
-        # Remove the ~-expanded one
-        >>> del files[1]
-        >>> files
-        ['/etc/git-buildpackage/gbp.conf', '.gbp.conf', 'debian/gbp.conf', '.git/gbp.conf']
-
+        >>> files_mangled = [file.replace(homedir, 'HOME') for file in files]
+        >>> files_mangled
+        ['/etc/git-buildpackage/gbp.conf', 'HOME/.gbp.conf', '%(top_dir)s/.gbp.conf', '%(top_dir)s/debian/gbp.conf', '%(git_dir)s/gbp.conf']
+        >>> files = GbpOptionParser.get_config_files(no_local=True)
+        >>> files_mangled = [file.replace(homedir, 'HOME') for file in files]
+        >>> files_mangled
+        ['/etc/git-buildpackage/gbp.conf', 'HOME/.gbp.conf']
         >>> os.environ['GBP_CONF_FILES'] = 'test1:test2'
         >>> GbpOptionParser.get_config_files()
         ['test1', 'test2']
@@ -303,9 +356,39 @@ class GbpOptionParser(OptionParser):
         """
         envvar = os.environ.get('GBP_CONF_FILES')
         files = envvar.split(':') if envvar else klass.def_config_files
-        return [ os.path.expanduser(f) for f in files ]
+        files = [os.path.expanduser(fname) for fname in files]
+        if no_local:
+            files = [fname for fname in files if fname.startswith('/')]
+        return files
 
-    def _parse_config_files(self):
+    def _read_config_file(self, parser, repo, filename, git_treeish):
+        """Read config file"""
+        str_fields = {}
+        if repo:
+            str_fields['git_dir'] = repo.git_dir
+            if not repo.bare:
+                str_fields['top_dir'] = repo.path
+
+        # Read per-tree config file
+        if repo and git_treeish and filename.startswith('%(top_dir)s/'):
+            with tempfile.TemporaryFile() as tmp:
+                relpath = filename.replace('%(top_dir)s/', '')
+                try:
+                    config = repo.show('%s:%s' % (git_treeish, relpath))
+                    tmp.writelines(config)
+                except GitRepositoryError:
+                    pass
+                tmp.seek(0)
+                parser.readfp(tmp)
+                return
+        try:
+            filename = filename % str_fields
+        except KeyError:
+            # Skip if filename wasn't expanded, i.e. we're not in git repo
+            return
+        parser.read(filename)
+
+    def parse_config_files(self, git_treeish=None):
         """
         Parse the possible config files and set appropriate values
         default values
@@ -315,25 +398,40 @@ class GbpOptionParser(OptionParser):
         self.config = dict(self.__class__.defaults)
         # Update with the values from the defaults section. This is needed
         # in case the config file doesn't have a [<command>] section at all
-        parser.read(self.config_files)
+        config_files = self.get_config_files()
+        try:
+            repo = GitRepository(".")
+        except GitRepositoryError:
+            repo = None
+        # Read all config files
+        for filename in config_files:
+            self._read_config_file(parser, repo, filename, git_treeish)
         self.config.update(dict(parser.defaults()))
 
-        if not (self.command.startswith('gbp-') or
-                self.command.startswith('git-')):
-            # Invoked as gbp <command> syntax, so parse the old sections
-            # of {gbp.git}-<command> for backward compatibility:
+        # Make sure we read any legacy sections prior to the real subcommands
+        # section i.e. read [gbp-pull] prior to [pull]
+        if (self.command.startswith('gbp-') or
+            self.command.startswith('git-')):
+            oldcmd = self.command
+            if parser.has_section(oldcmd):
+                self.config.update(dict(parser.items(oldcmd, raw=True)))
+            cmd = self.command[4:]
+        else:
             for prefix in ['gbp', 'git']:
                 oldcmd = '%s-%s' % (prefix, self.command)
                 if parser.has_section(oldcmd):
                     self.config.update(dict(parser.items(oldcmd, raw=True)))
+            cmd = self.command
 
         # Update with command specific settings
-        if parser.has_section(self.command):
-            self.config.update(dict(parser.items(self.command, raw=True)))
+        if parser.has_section(cmd):
+            # Don't use items() until we got rid of the compat sections
+            # since this pulls in the defaults again
+            self.config.update(dict(parser._sections[cmd].items()))
 
         for section in self.sections:
             if parser.has_section(section):
-                self.config.update(dict(parser.items(section, raw=True)))
+                self.config.update(dict(parser._sections[section].items()))
             else:
                 raise NoSectionError("Mandatory section [%s] does not exist."
                                      % section)
@@ -347,7 +445,8 @@ class GbpOptionParser(OptionParser):
         else:
             self.config['filter'] = []
 
-    def __init__(self, command, prefix='', usage=None, sections=[]):
+    def __init__(self, command, prefix='', usage=None, sections=[],
+                 git_treeish=None):
         """
         @param command: the command to build the config parser for
         @type command: C{str}
@@ -363,9 +462,15 @@ class GbpOptionParser(OptionParser):
         self.sections = sections
         self.prefix = prefix
         self.config = {}
-        self.config_files = self.get_config_files()
-        self._parse_config_files()
+        self.parse_config_files(git_treeish)
+        self.valid_options = []
+
+        if self.command.startswith('git-') or self.command.startswith('gbp-'):
+            prog = self.command
+        else:
+            prog = "gbp %s" % self.command
         OptionParser.__init__(self, option_class=GbpOption,
+                              prog=prog,
                               usage=usage, version='%s %s' % (self.command,
                                                               gbp_version))
 
@@ -412,38 +517,49 @@ class GbpOptionParser(OptionParser):
             default = self.config[option_name]
         return default
 
-    def add_config_file_option(self, option_name, dest, help=None, **kwargs):
+    @safe_option
+    def add_config_file_option(self, option_name, help=None, **kwargs):
         """
         set a option for the command line parser, the default is read from the config file
-        @param option_name: name of the option
-        @type option_name: string
-        @param dest: where to store this option
-        @type dest: string
-        @param help: help text
-        @type help: string
+        param option_name: name of the option
+        type option_name: string
+        param help: help text
+        type help: string
         """
         if not help:
             help = self.help[option_name]
-        OptionParser.add_option(self, "--%s%s" % (self.prefix, option_name), dest=dest,
+        OptionParser.add_option(self, "--%s%s" % (self.prefix, option_name),
                                 default=self.get_default(option_name, **kwargs),
                                 help=help % self.config, **kwargs)
 
-    def add_boolean_config_file_option(self, option_name, dest):
+    def add_boolean_config_file_option(self, option_name, dest=None):
         self.add_config_file_option(option_name=option_name, dest=dest, action="store_true")
         neg_help = "negates '--%s%s'" % (self.prefix, option_name)
         self.add_config_file_option(option_name="no-%s" % option_name, dest=dest, help=neg_help, action="store_false")
 
+    def get_config_file_value(self, option_name):
+        """
+        Query a single interpolated config file value.
+
+        @param option_name: the config file option to look up
+        @type option_name: string
+        @returns: The config file option value or C{None} if it doesn't exist
+        @rtype: C{str} or C{None}
+        """
+        return self.config.get(option_name)
+
 
 class GbpOptionGroup(OptionGroup):
+    @safe_option
     def add_config_file_option(self, option_name, dest, help=None, **kwargs):
         """
         set a option for the command line parser, the default is read from the config file
-        @param option_name: name of the option
-        @type option_name: string
-        @param dest: where to store this option
-        @type dest: string
-        @param help: help text
-        @type help: string
+        param option_name: name of the option
+        type option_name: string
+        param dest: where to store this option
+        type dest: string
+        param help: help text
+        type help: string
         """
         if not help:
             help = self.parser.help[option_name]
@@ -464,7 +580,158 @@ class GbpOptionParserDebian(GbpOptionParser):
     defaults = dict(GbpOptionParser.defaults)
     defaults.update( {
                        'builder'            : 'debuild -i -I',
-                       'cleaner'            : 'debuild -d clean',
+                       'cleaner'            : '/bin/true',
+                       'export-dir'         : '',
                      } )
 
+class GbpOptionParserRpm(GbpOptionParser):
+    """
+    Handles commandline options and parsing of config files for rpm tools
+    """
+    defaults = dict(GbpOptionParser.defaults)
+    defaults.update({
+            'tmp-dir'                   : '/var/tmp/gbp/',
+            'vendor'                    : 'vendor',
+			'native'                    : 'auto',
+			'builder'                   : 'rpmbuild',
+            'cleaner'                   : '/bin/true',
+			'merge'                     : 'False',
+			'packaging-dir'             : '',
+            'packaging-branch'          : 'master',		
+            'packaging-tag'             : '%(vendor)s/%(version)s',
+            'upstream-tag'              : 'upstream/%(upstreamversion)s',
+            'pq-branch'                 : 'development/%(branch)s',
+            'import-files'              : ['.gbp.conf',
+                                           'debian/gbp.conf'],
+            'spec-file'                 : 'auto',
+            
+            'export-dir'                : '../rpmbuild',
+            'ignore-untracked'          : 'False',
+            'rpmbuild-builddir'         : 'BUILD',
+            'rpmbuild-rpmdir'           : 'RPMS',
+            'rpmbuild-sourcedir'        : 'SOURCES',
+            'rpmbuild-specdir'          : 'SPECS',
+            'rpmbuild-srpmdir'          : 'SRPMS',
+            'rpmbuild-buildrootdir'     : 'BUILDROOT',
+            'patch-export'              : 'False',
+			'patch-export-ignore-path'  : '',
+            'patch-export-compress'     : '0',
+            'patch-export-squash-until' : '',
+			'pristine-tarball-name'     : 'auto',
+			'orig-prefix'               : 'auto',
+            'patch-import'              : 'True',
+            'spec-vcs-tag'              : '',
+                        
+            
+            'changelog-file'            : 'auto',
+            'changelog-revision'        : '',
+            'spawn-editor'              : 'always',
+            'editor-cmd'                : 'vim',
+            'meta-bts'                  : '(Close|Closes|Fixes|Fix)',
+                    })
+
+    help = dict(GbpOptionParser.help)
+    help.update({
+            
+            'vendor':
+                "Distribution vendor name, default is '%(vendor)s'",
+			'native':
+                "Treat this package as native, default is '%(native)s'",
+            'packaging-branch':
+                "Branch the packaging is being maintained on, rpm counterpart "
+                "of the 'debian-branch' option, default is "
+                "'%(packaging-branch)s'",
+            'packaging-dir':
+                "Subdir for RPM packaging files, default is '%(packaging-dir)s'",
+            'packaging-tag':
+                "Format string for packaging tags, RPM counterpart of the "
+                "'debian-tag' option, default is '%(packaging-tag)s'",
+            'pq-branch':
+                "format string for the patch-queue branch name, default is '%(pq-branch)s'",
+            'import-files':
+                "Comma-separated list of additional file(s) to import from "
+                "packaging branch. These will appear as one monolithic patch "
+                "in the pq/development branch. Default is %(import-files)s",
+            'spec-file':
+                "Spec file to use, 'auto' makes gbp to guess, other values "
+                "make the packaging-dir option to be ignored, default is "
+                "'%(spec-file)s'",
+            
+            'ignore-untracked':
+                "build with untracked files in the source tree, default is "
+                "'%(ignore-untracked)s'",
+            'patch-export':
+                "Create patches between upstream and export-treeish, default "
+                "is '%(patch-export)s'",
+            'patch-export-compress':
+                "Compress (auto-generated) patches larger than given number of "
+                "bytes, 0 never compresses, default is "
+                "'%(patch-export-compress)s'",
+            'patch-export-ignore-path':
+                "Exclude changes to path(s) matching regex, default is "
+                "'%(patch-export-ignore-path)s'",
+            'patch-export-squash-until':
+                "Squash commits (from upstream) until given tree-ish into one "
+                "big diff, format is '<commit_ish>[:<filename_base>]'. "
+                "Default is '%(patch-export-squash-until)s'",
+            'patch-import':
+                "Import patches to the packaging branch, default is "
+                "'%(patch-import)s'",
+            'spec-vcs-tag':
+                "Set/update the 'VCS:' tag in the spec file, empty value "
+                "removes the tag entirely, default is '%(spec-vcs-tag)s'",
+            'pristine-tarball-name':
+                "Filename to record to pristine-tar, set to 'auto' to not "
+                "mangle the file name, default is '%(pristine-tarball-name)s'",
+            'orig-prefix':
+                "Prefix (dir) to be used when generating/importing tarballs, "
+                "default is '%(orig-prefix)s'",
+            'changelog-file':
+                "Changelog file to be used, default is '%(changelog-file)s'",
+            'changelog-revision':
+                "Format string for the revision field in the changelog header. "
+                "If empty or not defined the default from packaging policy is "
+                "used.",
+            'editor-cmd':
+                "Editor command to use",
+            'git-author':
+                "Use name and email from git-config for the changelog header, "
+                "default is '%(git-author)s'",
+            'meta-bts':
+                "Meta tags for the bts commands, default is '%(meta-bts)s'",
+			'tmp-dir':
+                "Base directory under which temporary directories are "
+                "created, default is '%(tmp-dir)s'",
+                 })
+
+class GbpOptionParserBB(GbpOptionParserRpm):
+    """Commandline and config file option parser for the -bb commands"""
+    defaults = dict(GbpOptionParserRpm.defaults)
+    defaults.update( {
+                       'builder'                : 'bitbake',
+                       'export-dir'             : '',
+                       'meta-dir'               : '',
+                       'bb-file'                : '',
+                       'bb-vcs-info'            : '',
+                       'submit-tag'             : 'submit/%(target)s/%(nowtime)s',
+                       'target'                 : 'tizen',
+                     } )
+
+    help = dict(GbpOptionParserRpm.help)
+    help.update( {
+                   'meta-dir':
+                        "Subdir where bitbake meta files are stored, default "
+                        "is '%(meta-dir)s'",
+                   'bb-file':
+                        "Bitbake recipe file to build",
+                   'bb-vcs-info':
+                        "Format string for the VCS information automatically "
+                        "set in the recipe file, default is '%(bb-vcs-info)s'",
+                   'submit-tag':
+                        "Submit tag format, default is '%(submit-tag)s'",
+                   'target':
+                        "Submit target used in submit tag, default is "
+                        "'%(target)s'",
+                 } )
+
 # vim:et:ts=4:sw=4:et:sts=4:ai:set list listchars=tab\:»·,trail\:·:
