@@ -106,7 +106,7 @@ def format_changelog_entry(commit_info, options, last_commit=False):
     GitRepository.get_commit_info()).  If last_commit is not False,
     then this entry is the last one in the series."""
     entry = [commit_info['subject']]
-    body = commit_info['body']
+    body = commit_info['body'].splitlines()
     commitid = commit_info['id']
     (git_dch_cmds, body) = extract_git_dch_cmds(body, options)
 
@@ -115,11 +115,14 @@ def format_changelog_entry(commit_info, options, last_commit=False):
     if options.idlen:
         entry[0] = '[%s] ' % commitid[0:options.idlen] + entry[0]
 
-    (bts_cmds, body) = extract_bts_cmds(body, options)
-    (thanks, body) = extract_thanks_info(body, options)
+    bts_cmds = {}
+    thanks  = []
+    if options.meta:
+        (bts_cmds, body) = extract_bts_cmds(body, options)
+        (thanks, body) = extract_thanks_info(body, options)
     body = filter_ignore_rx_matches(body, options)
 
-    if options.full and not 'short' in git_dch_cmds:
+    if 'full' in git_dch_cmds or (options.full and not 'short' in git_dch_cmds):
         # Add all non-blank body lines.
         entry.extend([line for line in body if line.strip()])
     if thanks:
