@@ -20,9 +20,9 @@ import subprocess
 import os.path
 import re
 from collections import defaultdict
+import select
 
 import gbp.log as log
-from gbp.errors import GbpError
 from gbp.git.modifier import GitModifier
 from gbp.git.commit import GitCommit
 from gbp.git.errors import GitError
@@ -76,29 +76,47 @@ class GitRepository(object):
         all methods.
     """
 
-    def _check_bare(self):
-        """Check whether this is a bare repository"""
-        out, dummy, ret = self._git_inout('rev-parse', ['--is-bare-repository'],
-                                          capture_stderr=True)
+    def _check_dirs(self):
+        """Get top level dir and git meta data dir"""
+        out, dummy, ret = self._git_inout('rev-parse', ['--git-dir'],
+                                      capture_stderr=True)
         if ret:
             raise GitRepositoryError(
-                "Failed to get repository state at '%s'" % self.path)
-        self._bare = False if out.strip() != 'true' else True
-        self._git_dir = '' if self._bare else '.git'
+                "Failed to get repository git dir at '%s'" % self.path)
+
+        # Set git meta data dir
+        git_dir = out.strip()
+        if os.path.isabs(git_dir):
+            self._git_dir = git_dir
+        else:
+            self._git_dir = os.path.abspath(os.path.join(self.path, git_dir))
+
+        # Set top level dir correctly (in case repo was initialized
+        # from a subdir, for example)
+        if self.bare:
+            self._path = self._git_dir
+        else:
+            out, dummy, ret = self._git_inout('rev-parse', ['--show-toplevel'],
+                                              capture_stderr=True)
+            self._path = os.path.abspath(out.strip())
 
     def __init__(self, path):
         self._path = os.path.abspath(path)
-        self._bare = False
         try:
-            out, dummy, ret = self._git_inout('rev-parse', ['--show-cdup'],
+            # Check for bare repository
+            out, dummy, ret = self._git_inout('rev-parse', ['--is-bare-repository'],
                                               capture_stderr=True)
-            if ret or out.strip():
+            if ret:
                 raise GitRepositoryError("No Git repository at '%s': '%s'" % (self.path, out))
+            self._bare = False if out.strip() != 'true' else True
+
+            self._check_dirs()
+
         except GitRepositoryError:
             raise # We already have a useful error message
         except:
-            raise GitRepositoryError("No Git repository at '%s'" % self.path)
-        self._check_bare()
+            raise GitRepositoryError("No Git repository at '%s' (or any parent dir)" % self.path)
+
 
     @staticmethod
     def __build_env(extra_env):
@@ -141,7 +159,7 @@ class GitRepository(object):
         return output, popen.returncode
 
     def _git_inout(self, command, args, input=None, extra_env=None, cwd=None,
-                   capture_stderr=False):
+                   capture_stderr=False, capture_stdout=True):
         """
         Run a git command with input and return output
 
@@ -160,30 +178,104 @@ class GitRepository(object):
         """
         if not cwd:
             cwd = self.path
-        return self.__git_inout(command, args, input, extra_env, cwd, capture_stderr)
+        ret = 0
+        stdout = ''
+        stderr = ''
+        try:
+            for outdata in self.__git_inout(command, args, input, extra_env,
+                                            cwd, capture_stderr,
+                                            capture_stdout):
+                stdout += outdata[0]
+                stderr += outdata[1]
+        except GitRepositoryError as err:
+            ret = err.returncode
+        return stdout, stderr, ret
+
+    def _git_inout2(self, command, args, stdin=None, extra_env=None, cwd=None,
+                    capture_stderr=False):
+        """
+        Quite similar to C{_git_inout()} but returns stdout output of the git
+        command as a Python generator object, instead. Also, stderr is not
+        returned.
+
+        @note: The caller must consume the iterator that is returned, in order
+        to make sure that the git command runs and terminates.
+        """
+        if not cwd:
+            cwd = self.path
+        stderr = ''
+        try:
+            for outdata in self.__git_inout(command, args, stdin, extra_env,
+                                            cwd, capture_stderr, True):
+                stderr += outdata[1]
+                yield outdata[0]
+        except GitRepositoryError as err:
+            err.stderr = stderr
+            raise err
 
     @classmethod
-    def __git_inout(cls, command, args, input, extra_env, cwd, capture_stderr):
+    def __git_inout(cls, command, args, stdin, extra_env, cwd, capture_stderr,
+                    capture_stdout):
         """
-        As _git_inout but can be used without an instance
+        Run a git command without a a GitRepostitory instance.
+
+        Returns the git command output (stdout, stderr) as a Python generator
+        object.
+
+        @note: The caller must consume the iterator that is returned, in order
+        to make sure that the git command runs and terminates.
         """
+        def rm_polled_fd(file_obj, select_list):
+            file_obj.close()
+            select_list.remove(file_obj)
+
         cmd = ['git', command] + args
         env = cls.__build_env(extra_env)
+        stdout_arg = subprocess.PIPE if capture_stdout else None
+        stdin_arg = subprocess.PIPE if stdin else None
         stderr_arg = subprocess.PIPE if capture_stderr else None
-        stdin_arg = subprocess.PIPE if input else None
 
         log.debug(cmd)
         popen = subprocess.Popen(cmd,
                                  stdin=stdin_arg,
-                                 stdout=subprocess.PIPE,
+                                 stdout=stdout_arg,
                                  stderr=stderr_arg,
                                  env=env,
                                  close_fds=True,
                                  cwd=cwd)
-        (stdout, stderr) = popen.communicate(input)
-        return stdout, stderr, popen.returncode
+        out_fds = [popen.stdout] if capture_stdout else []
+        if capture_stderr:
+            out_fds.append(popen.stderr)
+        in_fds = [popen.stdin] if stdin else []
+        w_ind = 0
+        while out_fds or in_fds:
+            ready = select.select(out_fds, in_fds, [])
+            # Write in chunks of 512 bytes
+            if ready[1]:
+                try:
+                    popen.stdin.write(stdin[w_ind:w_ind+512])
+                except IOError:
+                    # Ignore, we want to read buffers to e.g. get error message
+                    # Git should give an error code so that we catch an error
+                    pass
+                w_ind += 512
+                if w_ind > len(stdin):
+                    rm_polled_fd(popen.stdin, in_fds)
+            # Read in chunks of 4k
+            stdout = popen.stdout.read(4096) if popen.stdout in ready[0] else ''
+            stderr = popen.stderr.read(4096) if popen.stderr in ready[0] else ''
+            if popen.stdout in ready[0] and not stdout:
+                rm_polled_fd(popen.stdout, out_fds)
+            if popen.stderr in ready[0] and not stderr:
+                rm_polled_fd(popen.stderr, out_fds)
+            yield stdout, stderr
 
-    def _git_command(self, command, args=[], extra_env=None):
+        if popen.wait():
+            err = GitRepositoryError('git-%s failed' % command)
+            err.returncode = popen.returncode
+            raise err
+
+    def _git_command(self, command, args=[], extra_env=None, interactive=False):
         """
         Execute git command with arguments args and environment env
         at path.
@@ -195,16 +287,19 @@ class GitRepository(object):
         @param extra_env: extra environment variables to set when running command
         @type extra_env: C{dict}
         """
+        capture_stdout = not interactive
         try:
             stdout, stderr, ret = self._git_inout(command=command,
                                                   args=args,
                                                   input=None,
                                                   extra_env=extra_env,
-                                                  capture_stderr=True)
+                                                  capture_stderr=True,
+                                                  capture_stdout=capture_stdout)
         except Exception as excobj:
             raise GitRepositoryError("Error running git %s: %s" % (command, excobj))
         if ret:
-            raise GitRepositoryError("Error running git %s: %s" % (command, stderr))
+            raise GitRepositoryError("Error running git %s: %s" %
+                                        (command, stderr.strip()))
 
 
     def _cmd_has_feature(self, command, feature):
@@ -231,6 +326,7 @@ class GitRepository(object):
         section_re = re.compile(r'^(?P<section>[A-Z].*)')
         option_re = re.compile(r'--?(?P<name>[a-zA-Z\-]+).*')
         optopt_re = re.compile(r'--\[(?P<prefix>[a-zA-Z\-]+)\]-?')
+        backspace_re = re.compile(".\b")
         man_section = None
         for line in help.splitlines():
             if man_section == "OPTIONS" and line.startswith('       -'):
@@ -248,7 +344,7 @@ class GitRepository(object):
             # Check man section
             match = section_re.match(line)
             if match:
-                man_section = match.group('section')
+                man_section = backspace_re.sub('', match.group('section'))
         return False
 
     @property
@@ -295,16 +391,18 @@ class GitRepository(object):
         args = GitArgs("-m", branch, newbranch)
         self._git_command("branch", args.args)
 
-    def create_branch(self, branch, rev=None):
+    def create_branch(self, branch, rev=None, force=False):
         """
         Create a new branch
 
         @param branch: the branch's name
         @param rev: where to start the branch from
+        @param force: reset branch HEAD to start point, if it already exists
 
         If rev is None the branch starts form the current HEAD.
         """
         args = GitArgs(branch)
+        args.add_true(force, '--force')
         args.add_true(rev, rev)
         self._git_command("branch", args.args)
 
@@ -441,6 +539,13 @@ class GitRepository(object):
         else:
             log.debug("Your git suite doesn't support --edit/--no-edit "
                       "option for git-merge ")
+
+        if (self._cmd_has_feature('merge', 'allow-unrelated-histories')):
+            args.add_cond(True, '--allow-unrelated-histories')
+        else:
+            log.debug("Your git suite doesn't support --allow-unrelated-histories "
+                      "option for git-merge ")
+
         args.add(commit)
         self._git_command("merge", args.args)
 
@@ -602,7 +707,8 @@ class GitRepository(object):
 
 #{ Tags
 
-    def create_tag(self, name, msg=None, commit=None, sign=False, keyid=None):
+    def create_tag(self, name, msg=None, commit=None, sign=False, keyid=None,
+                   annotate=False):
         """
         Create a new tag.
 
@@ -617,15 +723,18 @@ class GitRepository(object):
         @type sign: C{bool}
         @param keyid: the GPG keyid used to sign the tag
         @type keyid: C{str}
+        @param annotate: Create an annotated tag
+        @type annotate: C{bool}
         """
-        args = []
-        args += [ '-m', msg ] if msg else []
+        args = GitArgs()
+        args.add_true(msg, ['-m', msg])
         if sign:
-            args += [ '-s' ]
-            args += [ '-u', keyid ] if keyid else []
-        args += [ name ]
-        args += [ commit ] if commit else []
-        self._git_command("tag", args)
+            args.add('-s')
+            args.add_true(keyid, ['-u', keyid])
+        args.add_true(annotate, '-a')
+        args.add(name)
+        args.add_true(commit, commit)
+        self._git_command("tag", args.args, interactive=True)
 
     def delete_tag(self, tag):
         """
@@ -818,6 +927,39 @@ class GitRepository(object):
         if ret:
             raise GitRepositoryError("Can't execute repository clean: %s" % err)
 
+    def status(self, pathlist=None):
+        """
+        Check status of repository.
+
+        @param pathlist: List of paths to check status for
+        @type pathlist: C{list}
+        @return C{dict} of C{lists} of paths, where key is a git status flag.
+        @rtype C{dict}
+        """
+        options = GitArgs('--porcelain', '-z')
+        if pathlist:
+            for path in pathlist:
+                options.add(path)
+
+        out, err, ret = self._git_inout('status', options.args,
+                                        extra_env={'LC_ALL': 'C'})
+        if ret:
+            raise GitRepositoryError("Can't get repository status: %s" % err)
+
+        elements = out.split('\x00')
+        result = defaultdict(list)
+
+        while elements[0] != '':
+            element = elements.pop(0)
+            status = element[:2]
+            filepath = element[3:]
+            # Expect to have two filenames for renames and copies
+            if status[0] in ['R', 'C']:
+                filepath = elements.pop(0) + '\x00' + filepath
+            result[status].append(filepath)
+
+        return result
+
     def is_empty(self):
         """
         Is the repository empty?
@@ -843,10 +985,11 @@ class GitRepository(object):
         args = GitArgs("--quiet", "--verify")
         args.add_cond(short, '--short=%d' % short)
         args.add(name)
-        sha, ret = self._git_getoutput('rev-parse', args.args)
+        sha, stderr, ret = self._git_inout('rev-parse', args.args,
+                                            capture_stderr=True)
         if ret:
             raise GitRepositoryError("revision '%s' not found" % name)
-        return self.strip_sha1(sha[0], short)
+        return self.strip_sha1(sha.splitlines()[0], short)
 
     @staticmethod
     def strip_sha1(sha1, length=0):
@@ -1060,10 +1203,25 @@ class GitRepository(object):
         @deprecated: Use get_remotes() instead
 
         @return: remote repositories
-        @rtype: C{list} of C{str}
+        @rtype: C{dict} of C{list} of C{str}
         """
-        out = self._git_getoutput('remote')[0]
-        return [ remote.strip() for remote in out ]
+        stdout, stderr, ret = self._git_inout('remote', ['-v'],
+                                              capture_stderr=True)
+        if ret:
+            raise GitRepositoryError('Failed to get remotes: %s' % stderr)
+
+        remotes = {}
+        for rem in stdout.splitlines():
+            name, url_urltype = rem.split('\t', 1)
+            url, urltype = url_urltype.rsplit(' ', 1)
+            urltype = urltype.strip('()')
+            if not name in remotes:
+                remotes[name] = ['']
+            if urltype == 'fetch':
+                remotes[name][0] = url
+            else:
+                remotes[name].append(url)
+        return remotes
 
     def has_remote_repo(self, name):
         """
@@ -1197,7 +1355,7 @@ class GitRepository(object):
 
 #{ Files
 
-    def add_files(self, paths, force=False, index_file=None, work_tree=None):
+    def add_files(self, paths, force=False, untracked=True, index_file=None, work_tree=None):
         """
         Add files to a the repository
 
@@ -1205,15 +1363,17 @@ class GitRepository(object):
         @type paths: list or C{str}
         @param force: add files even if they would be ignored by .gitignore
         @type force: C{bool}
+        @param untracked: add also previously untracked files
+        @type untracked: C{bool}
         @param index_file: alternative index file to use
         @param work_tree: alternative working tree to use
         """
         extra_env = {}
 
-        if isinstance(paths, basestring):
-            paths = [ paths ]
-
-        args = [ '-f' ] if force else []
+        args = GitArgs()
+        args.add_true(force, '-f')
+        args.add_cond(untracked, '-A', '-u')
+        args.add(paths)
 
         if index_file:
             extra_env['GIT_INDEX_FILE'] =  index_file
@@ -1221,7 +1381,7 @@ class GitRepository(object):
         if work_tree:
             extra_env['GIT_WORK_TREE'] = work_tree
 
-        self._git_command("add", args + paths, extra_env)
+        self._git_command("add", args.args, extra_env)
 
     def remove_files(self, paths, verbose=False):
         """
@@ -1286,16 +1446,23 @@ class GitRepository(object):
         if not ret:
             return self.strip_sha1(sha1)
         else:
-            raise GbpError("Failed to hash %s: %s" % (filename, stderr))
+            raise GitRepositoryError("Failed to hash %s: %s" % (filename,
+                                                                stderr))
 #}
 
 #{ Comitting
 
-    def _commit(self, msg, args=[], author_info=None):
+    def _commit(self, msg, args=[], author_info=None,
+                committer_info=None, edit=False):
         extra_env = author_info.get_author_env() if author_info else None
-        self._git_command("commit", ['-q', '-m', msg] + args, extra_env=extra_env)
+        if committer_info:
+            extra_env.update(committer_info.get_committer_env())
+        default_args = ['-q', '-m', msg] + (['--edit'] if edit else [])
+        self._git_command("commit", default_args + args, extra_env=extra_env,
+                          interactive=edit)
 
-    def commit_staged(self, msg, author_info=None, edit=False):
+    def commit_staged(self, msg, author_info=None, edit=False,
+                      committer_info=None):
         """
         Commit currently staged files to the repository
 
@@ -1305,10 +1472,11 @@ class GitRepository(object):
         @type author_info: L{GitModifier}
         @param edit: whether to spawn an editor to edit the commit info
         @type edit: C{bool}
+        @param committer_info: committer information
+        @type committer_info: L{GitModifier}
         """
-        args = GitArgs()
-        args.add_true(edit,  '--edit')
-        self._commit(msg=msg, args=args.args, author_info=author_info)
+        self._commit(msg=msg, author_info=author_info,
+                     committer_info=committer_info, edit=edit)
 
     def commit_all(self, msg, author_info=None, edit=False):
         """
@@ -1318,11 +1486,10 @@ class GitRepository(object):
         @param author_info: authorship information
         @type author_info: L{GitModifier}
         """
-        args = GitArgs('-a')
-        args.add_true(edit,  '--edit')
-        self._commit(msg=msg, args=args.args, author_info=author_info)
+        self._commit(msg=msg, args=['-a'], author_info=author_info, edit=edit)
 
-    def commit_files(self, files, msg, author_info=None):
+    def commit_files(self, files, msg, author_info=None, committer_info=None,
+                     edit=False):
         """
         Commit the given files to the repository
 
@@ -1332,10 +1499,33 @@ class GitRepository(object):
         @type msg: C{str}
         @param author_info: authorship information
         @type author_info: L{GitModifier}
+        @param committer_info: committer information
+        @type committer_info: L{GitModifier}
+        @param edit: whether to spawn an editor to edit the commit info
+        @type edit: C{bool}
         """
-        if isinstance(files, basestring):
-            files = [ files ]
-        self._commit(msg=msg, args=files, author_info=author_info)
+        args = GitArgs('--')
+        args.add(files)
+        self._commit(msg=msg, args=args.args, author_info=author_info,
+                     committer_info=committer_info, edit=edit)
+
+    def create_tree(self, unpack_dir):
+        """
+        Create a tree object out of a directory content
+
+        @param unpack_dir: content to add
+        @type unpack_dir: C{str}
+        @return: the tree object hash
+        @rtype: C{str}
+        """
+        git_index_file = os.path.join(self.path, self._git_dir, 'gbp_index')
+        try:
+            os.unlink(git_index_file)
+        except OSError:
+            pass
+        self.add_files('.', force=True, index_file=git_index_file,
+                       work_tree=unpack_dir)
+        return self.write_tree(git_index_file)
 
     def commit_dir(self, unpack_dir, msg, branch, other_parents=None,
                    author={}, committer={}, create_missing_branch=False):
@@ -1359,15 +1549,7 @@ class GitRepository(object):
             doesn't already exist.
         @type create_missing_branch: C{bool}
         """
-
-        git_index_file = os.path.join(self.path, self._git_dir, 'gbp_index')
-        try:
-            os.unlink(git_index_file)
-        except OSError:
-            pass
-        self.add_files('.', force=True, index_file=git_index_file,
-                       work_tree=unpack_dir)
-        tree = self.write_tree(git_index_file)
+        tree = self.create_tree(unpack_dir)
 
         if branch:
             try:
@@ -1408,7 +1590,7 @@ class GitRepository(object):
         @param parents: parents of this commit
         @param author: authorship information
         @type author: C{dict} with keys 'name' and 'email' or L{GitModifier}
-        @param committer: comitter information
+        @param committer: committer information
         @type committer: C{dict} with keys 'name' and 'email'
         """
         extra_env = {}
@@ -1430,7 +1612,7 @@ class GitRepository(object):
         if not ret:
             return self.strip_sha1(sha1)
         else:
-            raise GbpError("Failed to commit tree: %s" % stderr)
+            raise GitRepositoryError("Failed to commit tree: %s" % stderr)
 
 #{ Commit Information
 
@@ -1529,7 +1711,8 @@ class GitRepository(object):
         """
         commit_sha1 = self.rev_parse("%s^0" % commitish)
         args = GitArgs('--pretty=format:%an%x00%ae%x00%ad%x00%cn%x00%ce%x00%cd%x00%s%x00%f%x00%b%x00',
-                       '-z', '--date=raw', '--name-status', commit_sha1)
+                       '-z', '--date=raw', '--no-renames', '--name-status',
+                       commit_sha1)
         out, err, ret =  self._git_inout('show', args.args)
         if ret:
             raise GitRepositoryError("Unable to retrieve commit info for %s"
@@ -1629,7 +1812,7 @@ class GitRepository(object):
             options.add('--stat=%s' % stat)
         options.add_true(summary, '--summary')
         options.add_true(text, '--text')
-        options.add_true(ignore_submodules, '--ignore-submodules=all')
+        options.add_true(ignore_submodules, '--ignore-submodules')
         options.add(obj1)
         options.add_true(obj2, obj2)
         if paths:
@@ -1638,9 +1821,37 @@ class GitRepository(object):
         if ret:
             raise GitRepositoryError("Git diff failed")
         return output
+
+    def diff_status(self, obj1, obj2):
+        """
+        Get file-status of two git repository objects
+
+        @param obj1: first object
+        @type obj1: C{str}
+        @param obj2: second object
+        @type obj2: C{str}
+        @return: name-status
+        @rtype: C{defaultdict} of C{str}
+        """
+        options = GitArgs('--name-status', '-z', obj1, obj2)
+        output, stderr, ret = self._git_inout('diff', options.args)
+
+        elements = output.split('\x00')
+        result = defaultdict(list)
+
+        while elements[0] != '':
+            status = elements.pop(0)[0]
+            filepath = elements.pop(0)
+            # Expect to have two filenames for renames and copies
+            if status in ['R', 'C']:
+                result[status].append(filepath)
+                filepath = elements.pop(0)
+            result[status].append(filepath)
+
+        return result
 #}
 
-    def archive(self, format, prefix, output, treeish, **kwargs):
+    def archive(self, format, prefix, output, treeish, paths=None):
         """
         Create an archive from a treeish
 
@@ -1648,17 +1859,30 @@ class GitRepository(object):
         @type format: C{str}
         @param prefix: prefix to prepend to each filename in the archive
         @type prefix: C{str}
-        @param output: the name of the archive to create
-        @type output: C{str}
+        @param output: the name of the archive to create, empty string or
+            C{None} gives data as return value
+        @type output: C{str} or C{None}
         @param treeish: the treeish to create the archive from
         @type treeish: C{str}
-        @param kwargs: additional commandline options passed to git-archive
+        @param paths: List of paths to include in the archive
+        @type paths: C{list} of C{str}
+
+        @return: archive data as a generator object
+        @rtype: C{None} or C{generator} of C{str}
         """
-        args = [ '--format=%s' % format, '--prefix=%s' % prefix,
-                 '--output=%s' % output, treeish ]
-        out, ret = self._git_getoutput('archive', args, **kwargs)
-        if ret:
-            raise GitRepositoryError("Unable to archive %s" % treeish)
+        args = GitArgs('--format=%s' % format, '--prefix=%s' % prefix)
+        args.add_true(output, '--output=%s' % output)
+        args.add(treeish)
+        args.add("--")
+        args.add_cond(paths, paths)
+
+        if output:
+            out, err, ret = self._git_inout('archive', args.args)
+            if ret:
+                raise GitRepositoryError("Unable to archive %s: %s" % (treeish,
+                                                                       err))
+        else:
+            return self._git_inout2('archive', args.args)
 
     def collect_garbage(self, auto=False):
         """
@@ -1672,18 +1896,23 @@ class GitRepository(object):
 
 #{ Submodules
 
-    def has_submodules(self):
+    def has_submodules(self, treeish=None):
         """
         Does the repo have any submodules?
 
+        @param treeish: look into treeish
+        @type treeish: C{str}
         @return: C{True} if the repository has any submodules, C{False}
             otherwise
         @rtype: C{bool}
         """
-        if os.path.exists(os.path.join(self.path, '.gitmodules')):
+        if treeish:
+            try:
+                self.show('%s:.gitmodules' % treeish)
+            except GitRepositoryError:
+                return False
             return True
-        else:
-            return False
+        return os.path.exists(os.path.join(self.path, '.gitmodules'))
 
 
     def add_submodule(self, repo_path):
@@ -1775,18 +2004,20 @@ class GitRepository(object):
         try:
             if not os.path.exists(abspath):
                 os.makedirs(abspath)
+            stderr = ''
             try:
-                stdout, stderr, ret = klass.__git_inout(command='init',
-                                                        args=args.args,
-                                                        input=None,
-                                                        extra_env=None,
-                                                        cwd=abspath,
-                                                        capture_stderr=True)
+                for out in klass.__git_inout(command='init',
+                                             args=args.args,
+                                             stdin=None,
+                                             extra_env=None,
+                                             cwd=abspath,
+                                             capture_stderr=True,
+                                             capture_stdout=True):
+                    stderr += out[1]
+            except GitRepositoryError:
+                raise GitRepositoryError("Error running git init: %s" % stderr)
             except Exception as excobj:
                 raise GitRepositoryError("Error running git init: %s" % excobj)
-            if ret:
-                raise GitRepositoryError("Error running git init: %s" % stderr)
-
             if description:
                 with open(os.path.join(abspath, git_dir, "description"), 'w') as f:
                     description += '\n' if description[-1] != '\n' else ''
@@ -1837,21 +2068,26 @@ class GitRepository(object):
         try:
             if not os.path.exists(abspath):
                 os.makedirs(abspath)
-
+            stderr = ''
             try:
-                stdout, stderr, ret = klass.__git_inout(command='clone',
-                                                        args=args.args,
-                                                        input=None,
-                                                        extra_env=None,
-                                                        cwd=abspath,
-                                                        capture_stderr=True)
+                for out in klass.__git_inout(command='clone',
+                                             args=args.args,
+                                             stdin=None,
+                                             extra_env=None,
+                                             cwd=abspath,
+                                             capture_stderr=True,
+                                             capture_stdout=True):
+                    stderr += out[1]
+            except GitRepositoryError:
+                raise GitRepositoryError("Error running git clone: %s" % stderr)
             except Exception as excobj:
                 raise GitRepositoryError("Error running git clone: %s" % excobj)
-            if ret:
-                raise GitRepositoryError("Error running git clone: %s" % stderr)
 
             if not name:
-                name = remote.rstrip('/').rsplit('/',1)[1]
+                try:
+                    name = remote.rstrip('/').rsplit('/',1)[1]
+                except IndexError:
+                    name = remote.split(':', 1)[1]
                 if (mirror or bare):
                     if not name.endswith('.git'):
                         name = "%s.git" % name
