@@ -20,9 +20,11 @@
 import os
 import sys
 import logging
-from logging import (DEBUG, INFO, WARNING, ERROR, CRITICAL, getLogger)
+from logging import DEBUG, INFO, WARNING, ERROR, CRITICAL
 import gbp.tristate
 
+# Initialize default logger
+LOGGER = logging.getLogger(__name__)
 
 COLORS = dict([('none', 0)] + zip(['black', 'red', 'green', 'yellow', 'blue',
                                    'magenta', 'cyan', 'white'], range(30, 38)))
@@ -52,7 +54,7 @@ class GbpStreamHandler(logging.StreamHandler):
     OFF_SEQ = "\033[0m"
 
     def __init__(self, stream=None, color='auto'):
-        super(GbpStreamHandler, self).__init__(stream)
+        logging.StreamHandler.__init__(self, stream)
         self._color = gbp.tristate.Tristate(color)
         self._color_scheme = DEFAULT_COLOR_SCHEME.copy()
         msg_fmt = "%(color)s%(name)s:%(levelname)s: %(message)s%(coloroff)s"
@@ -75,7 +77,7 @@ class GbpStreamHandler(logging.StreamHandler):
         """Check if to print in color or not"""
         if self._color.is_on():
             return True
-        elif self._color.is_auto():
+        elif self._color.is_auto() and hasattr(self.stream, 'isatty'):
             in_emacs = (os.getenv("EMACS") and
                         os.getenv("INSIDE_EMACS", "").endswith(",comint"))
             return self.stream.isatty() and not in_emacs
@@ -88,35 +90,41 @@ class GbpStreamHandler(logging.StreamHandler):
             record.color = self.COLOR_SEQ % self._color_scheme[record.levelno]
             record.coloroff = self.OFF_SEQ
         record.levelname = record.levelname.lower()
-        return super(GbpStreamHandler, self).format(record)
+        return logging.StreamHandler.format(self, record)
 
 
 class GbpLogger(logging.Logger):
     """Logger class for git-buildpackage"""
 
-    def __init__(self, name, color='auto', *args, **kwargs):
-        super(GbpLogger, self).__init__(name, *args, **kwargs)
-        self._default_handlers = [GbpStreamHandler(sys.stdout, color),
-                                  GbpStreamHandler(sys.stderr, color)]
-        self._default_handlers[0].addFilter(GbpFilter([DEBUG, INFO]))
-        self._default_handlers[1].addFilter(GbpFilter([WARNING, ERROR,
-                                                       CRITICAL]))
-        for hdlr in self._default_handlers:
+    def __init__(self, name, *args, **kwargs):
+        logging.Logger.__init__(self, name, *args, **kwargs)
+        self.default_handlers = []
+
+    def init_default_handlers(self, color='auto'):
+        """Initialize and set default handlers to logger"""
+        self.default_handlers = [GbpStreamHandler(sys.stdout, color),
+                                 GbpStreamHandler(sys.stderr, color)]
+        self.default_handlers[0].addFilter(GbpFilter([DEBUG, INFO]))
+        self.default_handlers[1].addFilter(GbpFilter([WARNING, ERROR,
+                                                      CRITICAL]))
+        for hdlr in self.default_handlers:
             self.addHandler(hdlr)
+        # We don't want to propagate as we add our own handlers
+        self.propagate = False
 
     def set_color(self, color):
         """Set/unset colorized output of the default handlers"""
-        for hdlr in self._default_handlers:
+        for hdlr in self.default_handlers:
             hdlr.set_color(color)
 
     def set_color_scheme(self, color_scheme={}):
         """Set the color scheme of the default handlers"""
-        for hdlr in self._default_handlers:
+        for hdlr in self.default_handlers:
             hdlr.set_color_scheme(color_scheme)
 
     def set_format(self, fmt):
         """Set the format of the default handlers"""
-        for hdlr in self._default_handlers:
+        for hdlr in self.default_handlers:
             hdlr.set_format(fmt)
 
 
@@ -124,10 +132,16 @@ def err(msg):
     """Logs a message with level ERROR on the GBP logger"""
     LOGGER.error(msg)
 
+def error(msg):
+    err(msg)
+
 def warn(msg):
     """Logs a message with level WARNING on the GBP logger"""
     LOGGER.warning(msg)
 
+def warning(msg):
+    warn(msg)
+
 def info(msg):
     """Logs a message with level INFO on the GBP logger"""
     LOGGER.info(msg)
@@ -156,8 +170,22 @@ def _parse_color_scheme(color_scheme=""):
             except KeyError: pass
     return scheme
 
+def getLogger(*args, **kwargs):
+    """Gbp-specific function"""
+    if not issubclass(logging.getLoggerClass(), GbpLogger):
+        logging.setLoggerClass(GbpLogger)
+    color = kwargs.pop('color') if 'color' in kwargs else 'auto'
+    logger = logging.getLogger(*args, **kwargs)
+    if hasattr(logger, 'default_handlers') and not logger.default_handlers:
+        logger.init_default_handlers(color)
+    return logger
+
 def setup(color, verbose, color_scheme=""):
     """Basic logger setup"""
+    # Initialize, if not done yet
+    if not isinstance(LOGGER, GbpLogger):
+        initialize()
+
     LOGGER.set_color(color)
     LOGGER.set_color_scheme(_parse_color_scheme(color_scheme))
     if verbose:
@@ -165,9 +193,8 @@ def setup(color, verbose, color_scheme=""):
     else:
         LOGGER.setLevel(INFO)
 
-
-# Initialize the module
-logging.setLoggerClass(GbpLogger)
-
-LOGGER = getLogger("gbp")
+def initialize():
+    """Initialize the logger module"""
+    global LOGGER
+    LOGGER = getLogger("gbp")
 
