@@ -17,27 +17,37 @@
 #
 # inspired by dom-git-checkout
 #
-"""clone a repo and set it up for gbp"""
+"""Clone a GIT repository and set it up for gbp"""
 
+import ConfigParser
 import sys
 import os, os.path
 from gbp.config import (GbpOptionParser, GbpOptionGroup)
-from gbp.deb.git import DebianGitRepository
-from gbp.git import (GitRepository, GitRepositoryError)
+from gbp.git import GitRepositoryError
 from gbp.errors import GbpError
 import gbp.log
+try:
+    from gbp.deb.git import DebianGitRepository as GitRepository
+except ImportError:
+    from gbp.rpm.git import RpmGitRepository as GitRepository
 
 
-def parse_args (argv):
-    parser = GbpOptionParser(command=os.path.basename(argv[0]), prefix='',
-                             usage='%prog [options] repository - clone a remote repository')
+def build_parser(name):
+    try:
+        parser = GbpOptionParser(command=os.path.basename(name), prefix='',
+                                 usage='%prog [options] repository - clone a remote repository')
+    except ConfigParser.ParsingError as err:
+        gbp.log.err(err)
+        return None
+
     branch_group = GbpOptionGroup(parser, "branch options", "branch tracking and layout options")
     parser.add_option_group(branch_group)
 
     branch_group.add_option("--all", action="store_true", dest="all", default=False,
                             help="track all branches, not only debian and upstream")
     branch_group.add_config_file_option(option_name="upstream-branch", dest="upstream_branch")
-    branch_group.add_config_file_option(option_name="debian-branch", dest="debian_branch")
+    branch_group.add_config_file_option(option_name="debian-branch", dest="packaging_branch")
+    branch_group.add_config_file_option(option_name="packaging-branch", dest="packaging_branch")
     branch_group.add_boolean_config_file_option(option_name="pristine-tar", dest="pristine_tar")
     branch_group.add_option("--depth", action="store", dest="depth", default=0,
                             help="git history depth (for creating shallow clones)")
@@ -47,10 +57,16 @@ def parse_args (argv):
     parser.add_config_file_option(option_name="color", dest="color", type='tristate')
     parser.add_config_file_option(option_name="color-scheme",
                                   dest="color_scheme")
+    return parser
+
+
+def parse_args (argv):
+    parser = build_parser(argv[0])
+    if not parser:
+        return None, None
 
     (options, args) = parser.parse_args(argv)
     gbp.log.setup(options.color, options.verbose, options.color_scheme)
-
     return (options, args)
 
 
@@ -58,6 +74,8 @@ def main(argv):
     retval = 0
 
     (options, args) = parse_args(argv)
+    if not options:
+        return 1
 
     if len(args) < 2:
         gbp.log.err("Need a repository to clone.")
@@ -74,8 +92,8 @@ def main(argv):
         pass
 
     try:
-        repo = DebianGitRepository.clone(clone_to, source, options.depth,
-                                         auto_name=auto_name)
+        repo = GitRepository.clone(clone_to, source, options.depth,
+                                   auto_name=auto_name)
         os.chdir(repo.path)
 
         # Reparse the config files of the cloned repository so we pick up the
@@ -91,7 +109,7 @@ def main(argv):
                     local != "HEAD":
                         repo.create_branch(local, remote)
         else: # only track gbp's default branches
-            branches = [ options.debian_branch, options.upstream_branch ]
+            branches = [ options.packaging_branch, options.upstream_branch ]
             if options.pristine_tar:
                 branches += [ repo.pristine_tar_branch ]
             gbp.log.debug('Will track branches: %s' % branches)
@@ -101,7 +119,7 @@ def main(argv):
                     not repo.has_branch(branch):
                         repo.create_branch(branch, remote)
 
-        repo.set_branch(options.debian_branch)
+        repo.set_branch(options.packaging_branch)
 
     except GitRepositoryError as err:
         gbp.log.err("Git command failed: %s" % err)
