@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 #
-# (C) 2007, 2008, 2009, 2010 Guido Guenther <agx@sigxcpu.org>
+# (C) 2007, 2008, 2009, 2010, 2013 Guido Guenther <agx@sigxcpu.org>
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation; either version 2 of the License, or
@@ -15,7 +15,7 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-"""Generate Debian changelog entries from git commit messages"""
+"""Generate Debian changelog entries from GIT commit messages"""
 
 import ConfigParser
 import os.path
@@ -64,17 +64,19 @@ def get_author_email(repo, use_git_config):
     return author, email
 
 
-def fixup_section(repo, git_author, options, dch_options):
+def fixup_section(repo, use_git_author, options, dch_options):
     """
-    Fixup the changelog header and trailer's comitter and email address
+    Fixup the changelog header and trailer's committer and email address
 
     It might otherwise point to the last git committer instead of the person
     creating the changelog
-    This apply --distribution and --urgency options passed to git-dch
+
+    This also applies --distribution and --urgency options passed to gbp dch
     """
-    author, email = get_author_email(repo, git_author)
+    author, email = get_author_email(repo, use_git_author)
     used_options = ['distribution', 'urgency']
-    header_opts = []
+    opts = []
+    mainttrailer_opts = [ '--nomainttrailer', '--mainttrailer', '-t' ]
 
     # This must not be done for snapshots or snapshots changelog entries
     # will not be concatenated
@@ -83,11 +85,17 @@ def fixup_section(repo, git_author, options, dch_options):
             val = getattr(options, opt)
             if val:
                 gbp.log.debug("Set header option '%s' to '%s'" % (opt, val))
-                header_opts.append("--%s=%s" % (opt, val))
+                opts.append("--%s=%s" % (opt, val))
     else:
         gbp.log.debug("Snapshot enabled: do not fixup options in header")
 
-    ChangeLog.spawn_dch(msg='', author=author, email=email, dch_options=dch_options+header_opts)
+    if use_git_author:
+        for opt in mainttrailer_opts:
+            if opt in dch_options:
+                break
+        else:
+            opts.append(mainttrailer_opts[0])
+    ChangeLog.spawn_dch(msg='', author=author, email=email, dch_options=dch_options+opts)
 
 
 def snapshot_version(version):
@@ -126,8 +134,8 @@ def mangle_changelog(changelog, cp, snapshot=''):
     """
     try:
         tmpfile = '%s.%s' % (changelog, snapshot)
-        cw = file(tmpfile, 'w')
-        cr = file(changelog, 'r')
+        cw = open(tmpfile, 'w')
+        cr = open(changelog, 'r')
 
         print >>cw, ("%(Source)s (%(MangledVersion)s) "
                      "%(Distribution)s; urgency=%(urgency)s\n" % cp)
@@ -153,9 +161,9 @@ def mangle_changelog(changelog, cp, snapshot=''):
         raise GbpError("Error mangling changelog %s" % e)
 
 
-def do_release(changelog, repo, cp, git_author, dch_options):
+def do_release(changelog, repo, cp, use_git_author, dch_options):
     """Remove the snapshot header and set the distribution"""
-    author, email = get_author_email(repo, git_author)
+    author, email = get_author_email(repo, use_git_author)
     (release, snapshot) = snapshot_version(cp['Version'])
     if snapshot:
         cp['MangledVersion'] = release
@@ -192,28 +200,46 @@ def parse_commit(repo, commitid, opts, last_commit=False):
     return entry, (author, email)
 
 
-def guess_snapshot_commit(cp, repo, options):
+def guess_documented_commit(cp, repo, tagformat):
     """
-    Guess the last commit documented in the changelog from the snapshot banner
-    or the last point the changelog was touched.
+    Guess the last commit documented in the changelog from the snapshot banner,
+    the last tagged version or the last point the changelog was touched.
+
+    @param cp: the changelog
+    @param repo: the git repository
+    @param tagformat: the format for Debian tags
+    @returns: the commit that was last documented in the changelog
+    @rtype: C{str}
+    @raises GbpError: In case we fail to find a commit to start at
     """
+    # Check for snapshot banner
     sr = re.search(snapshot_re, cp['Changes'])
     if sr:
         return sr.group('commit')
-    # If the current topmost changelog entry has already been tagged rely on
-    # the version information only. The upper level relies then on the version
-    # info anyway:
-    if repo.find_version(options.debian_tag, cp.version):
-        return None
-    # If we didn't find a snapshot header we look at the point the changelog
-    # was last touched.
+
+    # Check if the latest version in the changelog is already tagged. If
+    # so this is the last documented commit.
+    commit = repo.find_version(tagformat, cp.version)
+    if commit:
+        gbp.log.info("Found tag for topmost changelog version '%s'" % commit)
+        return commit
+
+    # Check when the changelog was last touched
     last = repo.get_commits(paths="debian/changelog", num=1)
     if last:
         gbp.log.info("Changelog last touched at '%s'" % last[0])
         return last[0]
+
+    # Changelog not touched yet
     return None
 
 
+def has_snapshot_banner(cp):
+    """Whether the changelog has a snapshot banner"""
+    sr = re.search(snapshot_re, cp['Changes'])
+    return True if sr else False
+
+
 def get_customizations(customization_file):
     if customization_file:
         execfile(customization_file,
@@ -265,20 +291,14 @@ def changelog_commit_msg(options, version):
     return options.commit_msg % dict(version=version)
 
 
-def main(argv):
-    ret = 0
-    changelog = 'debian/changelog'
-    until = 'HEAD'
-    found_snapshot_header = False
-    version_change = {}
-    branch = None
-
+def build_parser(name):
     try:
-        parser = GbpOptionParserDebian(command=os.path.basename(argv[0]), prefix='',
+        parser = GbpOptionParserDebian(command=os.path.basename(name),
                                        usage='%prog [options] paths')
     except ConfigParser.ParsingError as err:
         gbp.log.err(err)
-        return 1
+        return None
+
     range_group = GbpOptionGroup(parser, "commit range options",
                                  "which commits to add to the changelog")
     version_group = GbpOptionGroup(parser, "release & version number options",
@@ -296,9 +316,9 @@ def main(argv):
     parser.add_option_group(custom_group)
 
     parser.add_boolean_config_file_option(option_name = "ignore-branch", dest="ignore_branch")
-    naming_group.add_config_file_option(option_name="debian-branch", dest="debian_branch")
+    naming_group.add_config_file_option(option_name="debian-branch", dest="packaging_branch")
     naming_group.add_config_file_option(option_name="upstream-tag", dest="upstream_tag")
-    naming_group.add_config_file_option(option_name="debian-tag", dest="debian_tag")
+    naming_group.add_config_file_option(option_name="debian-tag", dest="packaging_tag")
     naming_group.add_config_file_option(option_name="snapshot-number", dest="snapshot_number",
                       help="expression to determine the next snapshot number, default is '%(snapshot-number)s'")
     parser.add_config_file_option(option_name="git-log", dest="git_log",
@@ -331,10 +351,9 @@ def main(argv):
                       help="Increment the Debian release number for a Debian Team upload, and add a Team upload changelog comment.")
     version_group.add_option("--security", dest="security", action="store_true", default=False,
                       help="Increment the Debian release number for a security upload and add a security upload changelog comment.")
-    version_group.add_boolean_config_file_option(option_name="git-author", dest="git_author")
+    version_group.add_boolean_config_file_option(option_name="git-author", dest="use_git_author")
     commit_group.add_boolean_config_file_option(option_name="meta", dest="meta")
-    commit_group.add_config_file_option(option_name="meta-closes", dest="meta_closes",
-                      help="Meta tags for the bts close commands, default is '%(meta-closes)s'")
+    commit_group.add_config_file_option(option_name="meta-closes", dest="meta_closes")
     commit_group.add_boolean_config_file_option(option_name="full", dest="full")
     commit_group.add_config_file_option(option_name="id-length", dest="idlen",
                       help="include N digits of the commit id in the changelog entry, default is '%(id-length)s'",
@@ -356,10 +375,31 @@ def main(argv):
                                         dest="customization_file",
                                         help=help_msg)
 
+
+    return parser
+
+
+def parse_args(argv):
+    parser = build_parser(argv[0])
+    if not parser:
+        return None, None
+
     (options, args) = parser.parse_args(argv[1:])
     gbp.log.setup(options.color, options.verbose, options.color_scheme)
     dch_options = process_options(options, parser)
     editor_cmd = process_editor_option(options)
+    return options, args, dch_options, editor_cmd
+
+def main(argv):
+    ret = 0
+    changelog = 'debian/changelog'
+    until = 'HEAD'
+    found_snapshot_banner = False
+    version_change = {}
+    branch = None
+
+
+    options, args, dch_options, editor_cmd = parse_args(argv)
 
     try:
         try:
@@ -374,9 +414,11 @@ def main(argv):
             if not options.ignore_branch:
                 raise
 
-        if options.debian_branch != branch and not options.ignore_branch:
-            gbp.log.err("You are not on branch '%s' but on '%s'" % (options.debian_branch, branch))
-            raise GbpError("Use --ignore-branch to ignore or --debian-branch to set the branch name.")
+        if options.packaging_branch != branch and not options.ignore_branch:
+            gbp.log.err("You are not on branch '%s' but on '%s'" %
+                        (options.packaging_branch, branch))
+            raise GbpError("Use --ignore-branch to ignore or --debian-branch "
+                           "to set the branch name.")
 
         source = DebianSource('.')
         cp = source.changelog
@@ -386,14 +428,15 @@ def main(argv):
         else:
             since = ''
             if options.auto:
-                since = guess_snapshot_commit(cp, repo, options)
+                since = guess_documented_commit(cp, repo, options.packaging_tag)
                 if since:
-                    gbp.log.info("Continuing from commit '%s'" % since)
-                    found_snapshot_header = True
+                    msg = "Continuing from commit '%s'" % since
                 else:
-                    gbp.log.info("Couldn't find snapshot header, using version info")
-            if not since:
-                since = repo.find_version(options.debian_tag, cp['Version'])
+                    msg = "Starting from first commit"
+                gbp.log.info(msg)
+                found_snapshot_banner = has_snapshot_banner(cp)
+            else: # Fallback: continue from last tag
+                since = repo.find_version(options.packaging_tag, cp['Version'])
                 if not since:
                     raise GbpError("Version %s not found" % cp['Version'])
 
@@ -420,10 +463,10 @@ def main(argv):
                 version_change['version'] = options.new_version
             # the user wants to force a new version
             add_section = True
-        elif cp['Distribution'] != "UNRELEASED" and not found_snapshot_header and commits:
+        elif cp['Distribution'] != "UNRELEASED" and not found_snapshot_banner and commits:
             # the last version was a release and we have pending commits
             add_section = True
-        elif options.snapshot and not found_snapshot_header:
+        elif options.snapshot and not found_snapshot_banner:
             # the user want to switch to snapshot mode
             add_section = True
         else:
@@ -471,11 +514,11 @@ def main(argv):
                            version=version_change,
                            dch_options=dch_options)
 
-        fixup_section(repo, git_author=options.git_author, options=options,
+        fixup_section(repo, use_git_author=options.use_git_author, options=options,
                       dch_options=dch_options)
 
         if options.release:
-            do_release(changelog, repo, cp, git_author=options.git_author,
+            do_release(changelog, repo, cp, use_git_author=options.use_git_author,
                        dch_options=dch_options)
         elif options.snapshot:
             (snap, version) = do_snapshot(changelog, repo, options.snapshot_number)
