@@ -15,63 +15,26 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-"""Import a new upstream version into a git repository"""
+"""Import a new upstream version into a GIT repository"""
 
 import ConfigParser
 import os
 import sys
-import tempfile
+import gbp.tmpfile as tempfile
 import gbp.command_wrappers as gbpc
 from gbp.deb import (DebianPkgPolicy, parse_changelog_repo)
+from gbp.deb.upstreamsource import DebianUpstreamSource
 from gbp.deb.uscan import (Uscan, UscanError)
 from gbp.deb.changelog import ChangeLog, NoChangeLogError
 from gbp.deb.git import (GitRepositoryError, DebianGitRepository)
 from gbp.config import GbpOptionParserDebian, GbpOptionGroup, no_upstream_branch_msg
 from gbp.errors import GbpError
+from gbp.format import format_msg
 import gbp.log
-from gbp.scripts.common.import_orig import (OrigUpstreamSource, cleanup_tmp_tree,
-                                            ask_package_name, ask_package_version,
-                                            repack_source, is_link_target)
-
-
-def prepare_pristine_tar(archive, pkg, version):
-    """
-    Prepare the upstream source for pristine tar import.
-
-    This checks if the upstream source is actually a tarball
-    and creates a symlink from I{archive}
-    to I{<pkg>_<version>.orig.tar.<ext>} so pristine-tar will
-    see the correct basename.
-
-    @param archive: the upstream source's name
-    @type archive: C{str}
-    @param pkg: the source package's name
-    @type pkg: C{str}
-    @param version: the upstream version number
-    @type version: C{str}
-    @rtype: C{str}
-    """
-    linked = False
-    if os.path.isdir(archive):
-        return None
-
-    ext = os.path.splitext(archive)[1]
-    if ext in ['.tgz', '.tbz2', '.tlz', '.txz' ]:
-        ext = ".%s" % ext[2:]
-
-    link = "../%s_%s.orig.tar%s" % (pkg, version, ext)
-
-    if os.path.basename(archive) != os.path.basename(link):
-        try:
-            if not is_link_target(archive, link):
-                os.symlink(os.path.abspath(archive), link)
-                linked = True
-        except OSError as err:
-                raise GbpError("Cannot symlink '%s' to '%s': %s" % (archive, link, err[1]))
-        return (link, linked)
-    else:
-        return (archive, linked)
-
+from gbp.pkg import compressor_opts
+from gbp.scripts.common.import_orig import (cleanup_tmp_tree, ask_package_name,
+						ask_package_version,
+						prepare_sources)
 
 def upstream_import_commit_msg(options, version):
     return options.import_msg % dict(version=version)
@@ -80,7 +43,7 @@ def upstream_import_commit_msg(options, version):
 def detect_name_and_version(repo, source, options):
     # Guess defaults for the package name and version from the
     # original tarball.
-    (guessed_package, guessed_version) = source.guess_version() or ('', '')
+    guessed_package, guessed_version = source.guess_version()
 
     # Try to find the source package name
     try:
@@ -91,7 +54,7 @@ def detect_name_and_version(repo, source, options):
             # Check the changelog file from the repository, in case
             # we're not on the debian-branch (but upstream, for
             # example).
-            cp = parse_changelog_repo(repo, options.debian_branch, 'debian/changelog')
+            cp = parse_changelog_repo(repo, options.packaging_branch, 'debian/changelog')
             sourcepackage = cp['Source']
         except NoChangeLogError:
             if options.interactive:
@@ -121,18 +84,17 @@ def detect_name_and_version(repo, source, options):
     return (sourcepackage, version)
 
 
-def find_source(options, args):
+def find_source(use_uscan, args):
     """Find the tarball to import - either via uscan or via command line argument
     @return: upstream source filename or None if nothing to import
     @rtype: string
     @raise GbpError: raised on all detected errors
     """
-    if options.uscan: # uscan mode
-        uscan = Uscan()
-
+    if use_uscan:
         if args:
             raise GbpError("you can't pass both --uscan and a filename.")
 
+        uscan = Uscan()
         gbp.log.info("Launching uscan...")
         try:
             uscan.scan()
@@ -153,10 +115,23 @@ def find_source(options, args):
     elif len(args) == 0:
         raise GbpError("No archive to import specified. Try --help.")
     else:
-        archive = OrigUpstreamSource(args[0])
+        archive = DebianUpstreamSource(args[0])
         return archive
 
 
+def pristine_tarball_name(source, pkg_name, pkg_version):
+    if source.is_tarball():
+        if source.compression:
+            comp_ext = '.' + compressor_opts[source.compression][1]
+        else:
+            comp_ext = ''
+    else:
+        # Need to repack and/or mangle filename if the archive is not
+        # pristine-tar-compatible -> we decide to create gz compressed tarball
+        comp_ext = '.gz'
+    return '%s_%s.orig.tar%s' % (pkg_name, pkg_version, comp_ext)
+
+
 def set_bare_repo_options(options):
     """Modify options for import into a bare repository"""
     if options.pristine_tar or options.merge:
@@ -167,13 +142,13 @@ def set_bare_repo_options(options):
         options.merge = False
 
 
-def parse_args(argv):
+def build_parser(name):
     try:
-        parser = GbpOptionParserDebian(command=os.path.basename(argv[0]), prefix='',
+        parser = GbpOptionParserDebian(command=os.path.basename(name), prefix='',
                                        usage='%prog [options] /path/to/upstream-version.tar.gz | --uscan')
     except ConfigParser.ParsingError as err:
         gbp.log.err(err)
-        return None, None
+        return None
 
     import_group = GbpOptionGroup(parser, "import options",
                       "pristine-tar and filtering")
@@ -189,12 +164,15 @@ def parse_args(argv):
     branch_group.add_option("-u", "--upstream-version", dest="version",
                       help="Upstream Version")
     branch_group.add_config_file_option(option_name="debian-branch",
-                      dest="debian_branch")
+                      dest="packaging_branch")
     branch_group.add_config_file_option(option_name="upstream-branch",
                       dest="upstream_branch")
     branch_group.add_option("--upstream-vcs-tag", dest="vcs_tag",
                             help="Upstream VCS tag add to the merge commit")
     branch_group.add_boolean_config_file_option(option_name="merge", dest="merge")
+    branch_group.add_boolean_config_file_option(
+                      option_name="create-missing-branches",
+                      dest="create_missing_branches")
 
     tag_group.add_boolean_config_file_option(option_name="sign-tags",
                       dest="sign_tags")
@@ -221,12 +199,20 @@ def parse_args(argv):
     parser.add_config_file_option(option_name="color", dest="color", type='tristate')
     parser.add_config_file_option(option_name="color-scheme",
                                   dest="color_scheme")
+    parser.add_config_file_option(option_name="tmp-dir", dest="tmp_dir")
 
     # Accepted for compatibility
     parser.add_option("--no-dch", dest='no_dch', action="store_true",
                       default=False, help="deprecated - don't use.")
     parser.add_option("--uscan", dest='uscan', action="store_true",
                       default=False, help="use uscan(1) to download the new tarball.")
+    return parser
+
+
+def parse_args(argv):
+    parser = build_parser(argv[0])
+    if not parser:
+        return None, None
 
     (options, args) = parser.parse_args(argv[1:])
     gbp.log.setup(options.color, options.verbose, options.color_scheme)
@@ -239,13 +225,15 @@ def parse_args(argv):
 
 def main(argv):
     ret = 0
-    tmpdir = ''
-    pristine_orig = None
-    linked = False
 
     (options, args) = parse_args(argv)
+    if not options:
+        return 1
+
+    tmpdir = tempfile.mkdtemp(dir=options.tmp_dir, prefix='import-orig_')
+
     try:
-        source = find_source(options, args)
+        source = find_source(options.uscan, args)
         if not source:
             return ret
 
@@ -259,9 +247,13 @@ def main(argv):
         is_empty = False if initial_branch else True
 
         if not repo.has_branch(options.upstream_branch) and not is_empty:
-            raise GbpError(no_upstream_branch_msg % options.upstream_branch)
+            if options.create_missing_branches:
+                gbp.log.info("Will create missing branch '%s'" %
+                             options.upstream_branch)
+            else:
+                raise GbpError(no_upstream_branch_msg % options.upstream_branch)
 
-        (sourcepackage, version) = detect_name_and_version(repo, source, options)
+        (pkg_name, version) = detect_name_and_version(repo, source, options)
 
         (clean, out) = repo.is_clean()
         if not clean and not is_empty:
@@ -271,22 +263,16 @@ def main(argv):
         if repo.bare:
             set_bare_repo_options(options)
 
-        if not source.is_dir():
-            tmpdir = tempfile.mkdtemp(dir='../')
-            source.unpack(tmpdir, options.filters)
-            gbp.log.debug("Unpacked '%s' to '%s'" % (source.path, source.unpacked))
-
-        if source.needs_repack(options):
-            gbp.log.debug("Filter pristine-tar: repacking '%s' from '%s'" % (source.path, source.unpacked))
-            (source, tmpdir)  = repack_source(source, sourcepackage, version, tmpdir, options.filters)
-
-        (pristine_orig, linked) = prepare_pristine_tar(source.path,
-                                                       sourcepackage,
-                                                       version)
+        # Prepare sources for importing
+        pristine_name = pristine_tarball_name(source, pkg_name, version)
+        prepare_pristine = pristine_name if options.pristine_tar else None
+        unpacked_orig, pristine_orig = prepare_sources(
+                source, pkg_name, version, prepare_pristine, options.filters,
+                options.filter_pristine_tar, None, tmpdir)
 
         # Don't mess up our repo with git metadata from an upstream tarball
         try:
-            if os.path.isdir(os.path.join(source.unpacked, '.git/')):
+            if os.path.isdir(os.path.join(unpacked_orig, '.git/')):
                 raise GbpError("The orig tarball contains .git metadata - giving up.")
         except OSError:
             pass
@@ -298,7 +284,7 @@ def main(argv):
             gbp.log.info("Importing '%s' to branch '%s'%s..." % (source.path,
                                                                  upstream_branch,
                                                                  filter_msg))
-            gbp.log.info("Source package is %s" % sourcepackage)
+            gbp.log.info("Source package is %s" % pkg_name)
             gbp.log.info("Upstream version is %s" % version)
 
             import_branch = [ options.upstream_branch, None ][is_empty]
@@ -309,17 +295,14 @@ def main(argv):
             else:
                 parents = None
 
-            commit = repo.commit_dir(source.unpacked,
-                                     msg=msg,
-                                     branch=import_branch,
-                                     other_parents=parents,
-                                     )
+            commit = repo.commit_dir(unpacked_orig,
+                        msg=msg,
+                        branch=import_branch,
+                        other_parents=parents,
+                        create_missing_branch=options.create_missing_branches)
 
-            if options.pristine_tar:
-                if pristine_orig:
-                    repo.pristine_tar.commit(pristine_orig, upstream_branch)
-                else:
-                    gbp.log.warn("'%s' not an archive, skipping pristine-tar" % source.path)
+            if options.pristine_tar and pristine_orig:
+                repo.pristine_tar.commit(pristine_orig, upstream_branch)
 
             tag = repo.version_to_tag(options.upstream_tag, version)
             repo.create_tag(name=tag,
@@ -331,8 +314,8 @@ def main(argv):
                 repo.create_branch(options.upstream_branch, rev=commit)
                 repo.force_head(options.upstream_branch, hard=True)
             elif options.merge:
-                gbp.log.info("Merging to '%s'" % options.debian_branch)
-                repo.set_branch(options.debian_branch)
+                gbp.log.info("Merging to '%s'" % options.packaging_branch)
+                repo.set_branch(options.packaging_branch)
                 try:
                     repo.merge(tag)
                 except GitRepositoryError:
@@ -347,25 +330,33 @@ def main(argv):
                         if cp.has_epoch():
                             epoch = '%s:' % cp.epoch
                     info = { 'version': "%s%s-1" % (epoch, version) }
-                    env = { 'GBP_BRANCH': options.debian_branch }
-                    gbpc.Command(options.postimport % info, extra_env=env, shell=True)()
+                    env = { 'GBP_BRANCH': options.packaging_branch }
+                    gbpc.Command(format_msg(options.postimport, info), extra_env=env, shell=True)()
             # Update working copy and index if we've possibly updated the
             # checked out branch
             current_branch = repo.get_branch()
             if current_branch in [ options.upstream_branch,
                                    repo.pristine_tar_branch]:
                 repo.force_head(current_branch, hard=True)
+            # Create symlink, if requested
+            if options.symlink_orig:
+                if source.is_tarball():
+                    link = os.path.join('..', pristine_name)
+                    if not (os.path.exists(link) and
+                            os.path.samefile(link, source.path)):
+                        gbp.log.info('Creating symlink to %s' % source.path)
+                        os.symlink(source.path, link)
+                else:
+                    gbp.log.warn('Orig source not a tarball, not symlinked')
+
         except (gbpc.CommandExecFailed, GitRepositoryError) as err:
             msg = err.__str__() if len(err.__str__()) else ''
             raise GbpError("Import of %s failed: %s" % (source.path, msg))
-    except GbpError as err:
+    except (GbpError, GitRepositoryError) as err:
         if len(err.__str__()):
             gbp.log.err(err)
         ret = 1
 
-    if pristine_orig and linked and not options.symlink_orig:
-        os.unlink(pristine_orig)
-
     if tmpdir:
         cleanup_tmp_tree(tmpdir)
 
