@@ -68,6 +68,7 @@ def test_add_files():
          - L{gbp.git.GitRepository.add_files}
          - L{gbp.git.GitRepository.commit_all}
          - L{gbp.git.GitRepository.is_clean}
+         - L{gbp.git.GitRepository.status}
 
     Properties tested:
          - L{gbp.git.GitRepository.head}
@@ -80,7 +81,12 @@ def test_add_files():
     False
     >>> repo.is_clean(ignore_untracked=True)[0]
     True
+    >>> repo.add_files('testfile', force=True, untracked=False)
+    >>> repo.status().items()
+    [('??', ['testfile'])]
     >>> repo.add_files(repo.path, force=True)
+    >>> repo.status().items()
+    [('A ', ['testfile'])]
     >>> repo.commit_all(msg="foo")
     >>> repo.is_clean()[0]
     True
@@ -104,6 +110,24 @@ def test_branch_master():
     'master'
     """
 
+def test_clean():
+    """
+    Remove untracked files from the working tree
+
+    Methods tested:
+         - L{gbp.git.GitRepository.clean}
+
+    >>> import gbp.git, shutil, os
+    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> shutil.copy(os.path.join(repo.path, ".git/HEAD"), \
+                                 os.path.join(repo.path, "testclean"))
+    >>> repo.clean(dry_run=True)
+    >>> repo.is_clean()[0]
+    False
+    >>> repo.clean(directories=True, force=True)
+    >>> repo.is_clean()[0]
+    True
+    """
 
 def test_create_branch():
     """
@@ -186,6 +210,7 @@ def test_set_upstream_branch():
     >>> os.makedirs(os.path.join(repo.git_dir, 'refs/remotes/origin'))
     >>> shutil.copy(os.path.join(repo.git_dir, 'refs/heads/master'), \
                     os.path.join(repo.git_dir, 'refs/remotes/origin/'))
+    >>> repo.add_remote_repo('origin', 'git://git.example.com/git/origin')
     >>> repo.set_upstream_branch('master', 'origin/master')
     >>> repo.get_upstream_branch('master')
     'origin/master'
@@ -195,7 +220,6 @@ def test_set_upstream_branch():
     >>> repo.set_upstream_branch('foo', 'origin/bla')
     Traceback (most recent call last):
     GitRepositoryError: Branch origin/bla doesn't exist!
-
     """
 
 def test_get_upstream_branch():
@@ -266,6 +290,12 @@ def test_describe():
     >>> repo.describe('HEAD', pattern='foo*', always=True, abbrev=16) == sha[:16]
     True
     >>> tag = repo.describe('HEAD', longfmt=True, abbrev=16) == 'tag2-0-g%s' % sha[:16]
+    >>> repo.delete_tag('tag2')
+    >>> repo.describe('HEAD', tags=True)
+    'tag'
+    >>> repo.describe('HEAD', tags=True, exact_match=True)
+    'tag'
+    >>> repo.create_tag('tag2', msg='foo')
     """
 
 def test_find_tag():
@@ -463,10 +493,25 @@ def test_diff():
     True
     >>> len(repo.diff('HEAD~1', 'HEAD', 'testfile')) > 3
     True
+    >>> len(repo.diff('HEAD~1', 'HEAD', 'testfile', text=True)) > 3
+    True
     >>> len(repo.diff('HEAD~1', 'HEAD', 'filenotexist')) == 0
     True
     """
 
+def test_diff_status():
+    """
+    Methods tested:
+        - L{gbp.git.GitRepository.diff_status}
+
+    >>> import gbp.git
+    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> repo.diff_status("HEAD", "HEAD")
+    defaultdict(<type 'list'>, {})
+    >>> repo.diff_status("HEAD~1", "HEAD")
+    defaultdict(<type 'list'>, {'M': ['testfile']})
+    """
+
 def test_mirror_clone():
     """
     Mirror a repository
@@ -532,8 +577,9 @@ def test_clone():
     >>> clone.get_merge_branch('bar') # None if no merge branch exists
     >>> clone.get_local_branches()
     ['bar', 'foo', 'master']
-    >>> clone.get_remote_repos()
-    ['origin']
+    >>> remotes = clone.get_remote_repos()
+    >>> {'origin': [repo_dir, repo_dir]} == remotes
+    True
     >>> clone.has_remote_repo('origin')
     True
     >>> clone.has_branch('origin/master', remote=True)
@@ -542,6 +588,28 @@ def test_clone():
     False
     """
 
+def test_get_remotes():
+    """
+    Merge a branch
+
+    Methods tested:
+         - L{gbp.git.GitRepository.get_remotes}
+
+    >>> import os
+    >>> import gbp.git.repository
+    >>> repo = gbp.git.repository.GitRepository(os.path.join(clone_dir, 'repo'))
+    >>> remotes = repo.get_remotes()
+    >>> len(remotes)
+    1
+    >>> origin = remotes['origin']
+    >>> origin.name
+    'origin'
+    >>> origin.fetch_url == repo_dir
+    True
+    >>> origin.push_urls == [repo_dir]
+    True
+    """
+
 def test_merge():
     """
     Merge a branch
@@ -569,6 +637,8 @@ def test_pull():
     >>> clone = gbp.git.GitRepository(d)
     >>> clone.set_branch('master')
     >>> clone.pull()
+    >>> clone.pull(all_remotes=True)
+    >>> clone.pull('origin', all_remotes=True)
     """
 
 def test_fetch():
@@ -589,11 +659,16 @@ def test_fetch():
     >>> clone.push()
     >>> clone.push('origin')
     >>> clone.push('origin', 'master')
+    >>> clone.push('origin', 'master', force=True)
     >>> clone.create_tag('tag3')
     >>> clone.push_tag('origin', 'tag3')
+    >>> clone.create_tag('tag4')
+    >>> clone.push('origin', 'master', tags=True)
     >>> clone.add_remote_repo('foo', repo_dir)
     >>> clone.fetch('foo')
     >>> clone.fetch('foo', tags=True)
+    >>> clone.fetch('foo', refspec='refs/heads/master')
+    >>> clone.fetch(all_remotes=True)
     >>> clone.remove_remote_repo('foo')
     """
 
@@ -609,7 +684,7 @@ def test_create_bare():
     >>> bare = gbp.git.GitRepository.create(bare_dir, bare=True, description="msg")
     >>> bare.path == bare_dir
     True
-    >>> bare.git_dir[:-1] == bare_dir
+    >>> bare.git_dir == bare_dir
     True
     >>> type(bare) == gbp.git.GitRepository
     True
@@ -630,7 +705,7 @@ def test_nonexistant():
     >>> bare = gbp.git.GitRepository("/does/not/exist")
     Traceback (most recent call last):
     ...
-    GitRepositoryError: No Git repository at '/does/not/exist'
+    GitRepositoryError: No Git repository at '/does/not/exist' (or any parent dir)
     """
 
 def test_create_noperm():
@@ -772,8 +847,11 @@ def test_make_tree():
     >>> head
     [['100644', 'blob', '19af7398c894bc5e86e17259317e4db519e9241f', 'testfile']]
     >>> head.append(['100644', 'blob', '19af7398c894bc5e86e17259317e4db519e9241f', 'testfile2'])
-    >>> repo.make_tree(head)
+    >>> newtree = repo.make_tree(head)
+    >>> newtree
     '745951810c9e22fcc6de9b23f05efd6ab5512123'
+    >>> repo.list_tree(newtree, recurse=False, paths='testfile')
+    [['100644', 'blob', '19af7398c894bc5e86e17259317e4db519e9241f', 'testfile']]
     """
 
 
@@ -810,6 +888,28 @@ def test_get_merge_base():
     GitRepositoryError: Failed to get common ancestor: fatal: Not a valid object name doesnotexist
     """
 
+def test_status():
+    r"""
+    Methods tested:
+        - L{gbp.git.GitRepository.status}
+
+    >>> import gbp.git, os, shutil
+    >>> repo = gbp.git.GitRepository(repo_dir)
+    >>> fname = os.path.join(repo.path, "test_status")
+    >>> shutil.copy(os.path.join(repo.path, ".git/HEAD"), fname)
+    >>> repo.status().items()
+    [('??', ['test_status'])]
+    >>> repo.status(['bla*']).items()
+    []
+    >>> repo.status(['te*']).items()
+    [('??', ['test_status'])]
+    >>> repo.add_files(repo.path, force=True)
+    >>> repo.commit_all(msg='added %s' % fname)
+    >>> _ = repo._git_inout('mv', [fname, fname + 'new'])
+    >>> repo.status().items()
+    [('R ', ['test_status\x00test_statusnew'])]
+    """
+
 def test_cmd_has_feature():
     r"""
     Methods tested:
